/***************************************************/
/* a routine from bigtest.c, G Edwards             */
/***************************************************/
#include <GL/glut.h>
#include <stdio.h>
#include "defines.h"
#include "externs.h"

/* trackBall - A simple trackball (not with proper rotations). */

/** A simple trackball with spin = left button
                           pan  = middle button
                           zoom = left + middle
   Doesn't have proper trackball rotation, ie axes which remain fixed in
   the scene. We should use the trackball code from 4Dgifts. */

#define STARTROTATE(x, y)     \
{                             \
    startMX = x;              \
    startMY = y;              \
}

#define STOPROTATE(x, y)      \
{                             \
    steadyXangle = varXangle; \
    steadyYangle = varYangle; \
}

#define STARTPAN(x, y)        \
{                             \
    startMX = x;              \
    startMY = y;              \
}

#define STOPPAN(x, y)         \
{                             \
    steadyX = varX;           \
    steadyY = varY;           \
}


#define STARTZOOM(x, y)       \
{                             \
    startMX = x;              \
    startMY = y;              \
}

#define STOPZOOM(x, y)        \
{                             \
    steadyZ = varZ;           \
}

bool leftDownUn ;  /* left button down ?  */
bool middleDownUn ;  /* middle button down ?  */

void trackBall(int mode, int button, int state, int x, int y)
{
  static int startMX = 0, startMY = 0;  /* initial mouse pos  */
  static int deltaMX = 0, deltaMY = 0;  /* initial mouse pos  */
  static float steadyXangle = 0.0, steadyYangle = 0.0;
  static float varXangle = 0.0, varYangle = 0.0;
  static float steadyX = 0.0, steadyY = 0.0, steadyZ = 0.0;
  static float varX = 0.0, varY = 0.0, varZ = 0.0;
  static GLint realy; /* OpenGL y coordinate position */

  switch (mode) {

  case RESET:
    steadyXangle = steadyYangle = steadyX = steadyY = steadyZ = 0.0;
    break;

  case MOUSEBUTTON:

    if (button == GLUT_LEFT_BUTTON && state == GLUT_DOWN && !middleDownUn) {
      STARTROTATE(x, y);
      leftDownUn = TRUE;
    } else if (button == GLUT_LEFT_BUTTON && state == GLUT_DOWN &&
      middleDownUn) {
      STOPPAN(x, y);
      STARTZOOM(x, y);
      leftDownUn = TRUE;
    } else if (button == GLUT_MIDDLE_BUTTON && state == GLUT_DOWN &&
      !leftDownUn) {
      STARTPAN(x, y);
      middleDownUn = TRUE;
    } else if (button == GLUT_MIDDLE_BUTTON && state == GLUT_DOWN &&
      leftDownUn) {
      STOPROTATE(x, y);
      STARTZOOM(x, y);
      middleDownUn = TRUE;
    } else if (state == GLUT_UP && button == GLUT_LEFT_BUTTON && !middleDownUn) {
      STOPROTATE(x, y);
      leftDownUn = FALSE;
    } else if (state == GLUT_UP && button == GLUT_LEFT_BUTTON && middleDownUn) {
      STOPZOOM(x, y);
      STARTROTATE(x, y);
      leftDownUn = FALSE;
    } else if (state == GLUT_UP && button == GLUT_MIDDLE_BUTTON && !leftDownUn) {
      STOPPAN(x, y);
      middleDownUn = FALSE;
    } else if (state == GLUT_UP && button == GLUT_MIDDLE_BUTTON && leftDownUn) {
      STOPZOOM(x, y);
      STARTROTATE(x, y);
      middleDownUn = FALSE;
    }
    break;

  case APPLY:


    if (leftDownUn && !middleDownUn ) {

      glTranslatef(steadyX, steadyY, steadyZ);
      glRotatef(varXangle, 0, 1, 0);
      glRotatef(varYangle, 1, 0, 0);
    }
    /* Middle button pan  */

    else if (middleDownUn && !leftDownUn ) {

      glTranslatef(varX, varY, steadyZ);
      glRotatef(steadyXangle, 0, 1, 0);
      glRotatef(steadyYangle, 1, 0, 0);
    }
    /* Left + middle zoom.  */

    else if (leftDownUn && middleDownUn ) {

      glTranslatef(steadyX, steadyY, varZ);
      glRotatef(steadyXangle, 0, 1, 0);
      glRotatef(steadyYangle, 1, 0, 0);
    }
    /* Nothing down.  */

    else {

      glTranslatef(steadyX, steadyY, steadyZ);
      glRotatef(steadyXangle, 0, 1, 0);
      glRotatef(steadyYangle, 1, 0, 0);
    }

    break;

  case MOUSEMOTION:

    deltaMX = x - startMX;
    deltaMY = startMY - y;


    if (leftDownUn && !middleDownUn) {

      varXangle = steadyXangle + deltaMX;
      varYangle = steadyYangle + deltaMY;

    } else if (middleDownUn && !leftDownUn) {
      varX = steadyX + deltaMX / 100.0;
      varY = steadyY + deltaMY / 100.0;
    } else if (leftDownUn && middleDownUn) {
      varZ = steadyZ - deltaMY / 50.0;
    }

    break;
  }
}


/**********************************************************************
* end of file
**********************************************************************/
