#include <GL/glut.h>
#include "externs.h"

/***** color scheme used in this scene *****/
#define CS390_POINTS_COLOR      0.0, 0.5, 0.0           /* dark green */
#define CS390_SURFACE_COLOR     1.0, 1.0, 0.0           /* yellow */

#define MAX_NUM_POINTS          100
#define MAX_NUM_KNOTS           40

/***** function declaration *****/
void draw_bspline_surface(int, int, int, int, int, int, float [], float [], float [][3]);
void draw_sphere(float, float, float, float);
void ReadMyData(char *, float [][3]);

/***** global variable *****/
float u = 0.0;          /** this value get changed from 0.0 to 1.0 by pressing
                         ** "left" and "right" arrows on keyboard. To control
                         ** the changing step, modify the "cs390_U_STEP" value
                         ** in "config.h" file. **/
float v = 0.0;          /** this value get changed from 0.0 to 1.0 by pressing
                         ** "up" and "down" arrows on keyboard. To control
                         ** the changing step, modify the "cs390_V_STEP" value
                         ** in "config.h" file. **/


/*------------------------------------------------------------------
   The following are some example data
   You should read in your own
------------------------------------------------------------------*/

int Udegree, Vdegree;    /* U,V degrees                   */
int Columns;      		/* points per row -- # of columns*/
int Rows;      		/* points per col -- # of rows   */
int num_Uknots;          /* number of knots on U          */
int num_Vknots;    	     /* number of knots on V          */
                        	
float knotU_vector[MAX_NUM_KNOTS];	/* knot vector on U    */
float knotV_vector[MAX_NUM_KNOTS]; /* knot vector on V    */

float points_data[MAX_NUM_POINTS][3];  /* ctrl points     */



/*** Your own rendering operation goes here ***/
void cs390_draw()
{
    	/* draw surface */
    	draw_bspline_surface(num_Uknots, num_Vknots,
                          Rows, Columns,
                          Udegree, Vdegree,
                          knotU_vector, knotV_vector,
                          points_data);

    	/***** other rendering code goes here *****/
    	glColor3f(1.0, 0.0, 0.0);
    	draw_sphere(-16.580, 16.299, 3.68, 1.35);
}


/*** Your initialization goes here ***/
/*** for example, read the data from the file ***/
/*** for a B-spline surface, on both U and V,
 *** you need to read in the number of control points,
 *** the number of knots in knot vector, the degree, along with
 *** the knot vector and the control points ***/
/*** "argc", "argv" are actual command line arguments passed in ***/

void cs390_init(int argc, char **argv)
{
    	/* write your own routine to read in data */
    	ReadMyData("input-0.dat", points_data );  /* read in data file */
}


/*****************************************************************************/
/*** Primitivie which draws a B-spline surface and its control points      ***/
/*** Param:     mU : the number of knots in knot vector on U
                mV : the number of knots in knot vector on V
                RPoints : the number of control points per row
                CPoints : the number of control points per column
                p : U degree
                q : V degree
                knot_u : the knot vector on U
                knot_v : the knot vector on V
                points_v : the control points array
 *** Note:
        Special relationship has to be satisfied to define a B-spline surface.
        i.e.
                mU = RPoints + p + 1
                mV = CPoints + q + 1
 ***    Make sure your data fit this requriement.
 *****************************************************************************/

void draw_bspline_surface(int mU, int mV, int Rows, int Columns, 
                          int p, int q,
                          float knot_u[], float knot_v[], 
                          float points_v[][3])
{
     int i, j;

    	/* draw the control points first */
    	glColor3f(CS390_POINTS_COLOR);
    	glPointSize(4);
    	glBegin(GL_POINTS);
     for(i=0; i < Rows; i++ )
        	for(j = 0; j < Columns; j++) {
            	glVertex3fv(&points_v[i*Columns+j][0]);
        	}
    	glEnd();

    	/* draw ctrl net step 1 */
    	for (i=0; i < Rows; i++ ) {
          glBegin(GL_LINE_STRIP);
        	for(j = 0; j < Columns; j++) {
          	glVertex3fv(&points_v[i*Columns+j][0]);
        	}
        	glEnd();
    	}

    	/* draw ctrl net step 2 */
    	for (j=0; j < Columns; j++ ) {
          glBegin(GL_LINE_STRIP);
        	for(i = 0; i < Rows; i++ ) {
          	glVertex3fv(&points_v[i*Columns+j][0]);
        	}
        	glEnd();
    	}

    	/* draw the B-spline surface */
    	if( ( mU != Rows + p + 1) || ( mV != Columns + q +1 )) {/* redundant check */
        	printf("Relation between #points, #knots and degree is");
        	printf("not satisfied!\n Can't draw B-spline surface\n");
        	return;
    	}
    	glColor3f(CS390_SURFACE_COLOR);
    	if ((Rows >= 2) && (Columns >=2 )) {
        	glEnable(GL_MAP2_VERTEX_3);
        	gluBeginSurface(theNurbs);
        	gluNurbsSurface(theNurbs,
                         mU,
                         knot_u,
                         mV,
                         knot_v,
                         Columns * 3,
                         3,
                         &points_v[0][0],
                         p + 1,
                         q + 1,
                         GL_MAP2_VERTEX_3);
        	gluEndSurface(theNurbs);
    	}
}

void draw_sphere(float x, float y, float z, float size)
{
    	glPushMatrix();
        	glTranslated(x, y, z);
        	gluSphere(theFinal, size, 30, 30);
    	glPopMatrix();
}


void ReadMyData(char *datFile, float points_data[][3])
{
     FILE *FdatFile;
     int i, j, PointsNum;

     if((FdatFile = fopen( datFile, "r")) == NULL){
          fprintf(stderr, "Can't read the data from %s!\n", datFile );
          exit(1);
     }

     fscanf(FdatFile, "%d %d %d %d", 
               &Rows, & Columns, &Udegree, &Vdegree);
          /* read in the # of rows and the # of columns */

     for ( i = 0; i < Rows* Columns; i ++ )  {
   		fscanf(FdatFile, "%f %f %f", 
               &points_data[i][0], &points_data[i][1],  &points_data[i][2]); 
     }
     num_Uknots = Rows + Udegree + 1;
     num_Vknots =  Columns + Vdegree + 1;
     for (i = 0; i < num_Uknots; i++)
          fscanf(FdatFile, "%f ", &knotU_vector[i]);
     for (i = 0; i < num_Vknots; i++)
          fscanf(FdatFile, "%f ", &knotV_vector[i]);
     fclose( FdatFile );
}


