#include <GL/glut.h>
#include "defines.h"
#include "colors.h"

/* Function prototypes and Global Variables */
#include "externs.h"
#include "util.h"
#include "config.h"
#include "TrackBall.h"

/***** global extern variables *****/
GLUnurbsObj *theNurbs;

/***** global variables used only in this file *****/
gmtu_Button b_exit = {10, 10, 55, 30, "Exit", GLUT_BITMAP_9_BY_15};
GLUquadricObj *theFinal;

/***** functions used in this file *****/
void mainwin_display();
void mainwin_reshape();
void mainwin_mouse();
void mainwin_motion();
void mainwin_special_keyboard(int, int, int);
void nurbsError(GLenum);


/* Main function */
int main(int argc, char **argv)
{
    int main_win;		/* useless windon ID */

    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);

    glutInitWindowSize(cs390_MAINWIN_WIDTH, cs390_MAINWIN_HEIGHT);
    cs390_init(argc, argv);
    main_win = glutCreateWindow("CS390 Display");
    glutDisplayFunc(mainwin_display); 
    glutReshapeFunc(mainwin_reshape);
    glutMouseFunc(mainwin_mouse);
    glutMotionFunc(mainwin_motion);
    glutSpecialFunc(mainwin_special_keyboard);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glClearColor(MAINWIN_BACKGROUND);

    /* initialize trackball  */
    trackBall(RESET, 0, 0, 0, 0);

    theFinal = gluNewQuadric();
    theNurbs = gluNewNurbsRenderer();
    gluNurbsProperty(theNurbs, GLU_SAMPLING_TOLERANCE, sampling_tolerance);
    gluNurbsProperty(theNurbs, GLU_DISPLAY_MODE, GLU_FILL );

    /* glut from Mesa-3.2.1 won't compile with this line
    gluNurbsCallback(theNurbs, GLU_ERROR,
			(GLvoid (CALLBACK*) ()) nurbsError);
    */

    glutMainLoop();
    return 0;
}

void mainwin_display()
{
    char string[50];

    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    glPushMatrix();
    glPushAttrib(GL_CURRENT_BIT);

    gmtuButtonEffect(1, 1, cs390_MAINWIN_WIDTH, cs390_MAINWIN_HEIGHT);
    glColor4f(1.0, 1.0, 0.0, 1.0);
    gmtuButtonDraw(&b_exit);
    
    /***** show u, v values here *****/
    glColor3f(0.8, 0.8, 0.0);
    sprintf(string, "u = %5.3f", u);
    gmtuDrawPara(string, cs390_MAINWIN_WIDTH / 2 - 120, 15, 1,
                        GLUT_BITMAP_9_BY_15);
    sprintf(string, "v = %5.3f", v);
    gmtuDrawPara(string, cs390_MAINWIN_WIDTH / 2 + 20, 15, 1,
                        GLUT_BITMAP_9_BY_15);

    glPushMatrix();
    glLoadIdentity();

    glTranslatef(cs390_MAINWIN_WIDTH / 2.0, cs390_MAINWIN_HEIGHT / 2.0, 0.0);
    glScalef(cs390_MAINWIN_WIDTH / cs390_X_RANGE,
               cs390_MAINWIN_HEIGHT / cs390_Y_RANGE, 1.0);

    /* trackball routine */
    trackBall(APPLY, 0, 0, 0, 0);

    cs390_draw();

    glPopMatrix();

    glPopAttrib();
    glPopMatrix();

    glutSwapBuffers();
    glFlush();
}


/**** light source at infinity *****/
GLfloat left_light_position[] = {-200.0, 100.0, 100.0, 0.0}, 
	right_light_position[] = {200.0, 100.0, 100.0, 0.0},
	back_light_position[] = {1.0, 1.0, -100.0, 0.0};
GLfloat red_light[] = {1.0, 0.0, 0.0, 1.0}, 
	green_light[] = {0.0, 1.0, 0.0, 1.0}, 
	white_light[] = {1.0, 1.0, 1.0, 1.0};
GLfloat  light_ambient[4]      = {0.2, 0.2, 0.2, 1.0};

void mainwin_reshape(int w, int h)
{
    cs390_MAINWIN_WIDTH = w;
    cs390_MAINWIN_HEIGHT = h;

    glLightfv(GL_LIGHT0, GL_AMBIENT,  light_ambient);
    glLightfv(GL_LIGHT0, GL_DIFFUSE,  white_light);
    glLightfv(GL_LIGHT0, GL_SPECULAR, white_light);
    glLightfv(GL_LIGHT0, GL_POSITION, left_light_position);
    glLightfv(GL_LIGHT1, GL_AMBIENT,  light_ambient);
    glLightfv(GL_LIGHT1, GL_DIFFUSE,  white_light);
    glLightfv(GL_LIGHT1, GL_SPECULAR, white_light);
    glLightfv(GL_LIGHT1, GL_POSITION, right_light_position);
    glLightfv(GL_LIGHT2, GL_AMBIENT,  light_ambient);
    glLightfv(GL_LIGHT2, GL_DIFFUSE,  white_light);
    glLightfv(GL_LIGHT2, GL_SPECULAR, white_light);
    glLightfv(GL_LIGHT2, GL_POSITION, back_light_position);
    glEnable(GL_LIGHT0);
    glEnable(GL_LIGHT1);
    glEnable(GL_LIGHT2);
    glEnable(GL_LIGHTING);

    glMaterialfv(GL_FRONT, GL_SHININESS, front_mat_shininess);
    glMaterialfv(GL_FRONT, GL_SPECULAR, front_mat_specular);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, front_mat_diffuse);
    glMaterialfv(GL_BACK, GL_SHININESS, back_mat_shininess);
    glMaterialfv(GL_BACK, GL_SPECULAR, back_mat_specular);
    glMaterialfv(GL_BACK, GL_DIFFUSE, back_mat_diffuse);

    glColorMaterial(GL_FRONT_AND_BACK, GL_DIFFUSE);
    glEnable(GL_COLOR_MATERIAL);
    glShadeModel(GL_SMOOTH);

    glEnable(GL_DEPTH_TEST);
    glEnable(GL_NORMALIZE);
    glEnable(GL_AUTO_NORMAL);
    glViewport(0, 0, cs390_MAINWIN_WIDTH, cs390_MAINWIN_HEIGHT);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(0, cs390_MAINWIN_WIDTH,
                0, cs390_MAINWIN_HEIGHT,
                -0.5 - cs390_Z_RANGE / 2, cs390_Z_RANGE / 2 + 0.5);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
}

void mainwin_mouse(int button, int state, int x, int y)
{
    int oldy = y;

    y = cs390_MAINWIN_HEIGHT - y;       /* change coordinate !!!!! */

    if(button == GLUT_LEFT_BUTTON && state == GLUT_UP) {
        if(gmtuButtonIn(&b_exit, x, y) == TRUE )
            exit(0);
     }

    trackBall(MOUSEBUTTON, button, state, x, y);
}

void mainwin_motion(int x, int y)
{
    trackBall(MOUSEMOTION, 0, 0, x, y);
    glutPostRedisplay();
}

void mainwin_special_keyboard(int key, int x, int y)
{
    switch(key) {
      case GLUT_KEY_LEFT:
	u -= cs390_U_STEP;
	if(u < 0.0 + cs390_U_STEP / 8)	u = 0.0;
	glutPostRedisplay();
	break;
      case GLUT_KEY_RIGHT:
	u += cs390_U_STEP;
	if(u > 1.0 - cs390_U_STEP / 8)	u = 1.0;
	glutPostRedisplay();
	break;
      case GLUT_KEY_UP:
	v -= cs390_V_STEP;
        if(v < 0.0 + cs390_V_STEP / 8)  v = 0.0;
        glutPostRedisplay();
        break;
      case GLUT_KEY_DOWN:
	v += cs390_V_STEP;
        if(v > 1.0 - cs390_V_STEP / 8)  v = 1.0;
        glutPostRedisplay();
        break;
      default:
	break;		/* nop */
    }
}

void nurbsError(GLenum errorCode)
{
    const char *estring;

    estring = gluErrorString(errorCode);
    fprintf(stderr, "Nurbs Error: %s!\n", estring);
}


/* ---------  end of file  -------------  */


